@@ -29,27 +29,34 @@ from cryptography.hazmat.primitives import hashes
 from cryptography.hazmat.primitives.twofactor import totp as crypto_totp
 from oslo_log import log
 from oslo_utils import timeutils
-import six
 
 from keystone.auth import plugins
 from keystone.auth.plugins import base
-from keystone.common import dependency
+from keystone.common import provider_api
+import keystone.conf
 from keystone import exception
 from keystone.i18n import _
 
 
+CONF = keystone.conf.CONF
+
 METHOD_NAME = 'totp'
 
 LOG = log.getLogger(__name__)
+PROVIDERS = provider_api.ProviderAPIs
 
 
-def _generate_totp_passcode(secret):
+PASSCODE_LENGTH = 6
+PASSCODE_TIME_PERIOD = 30
+
+
+def _generate_totp_passcodes(secret, included_previous_windows=0):
     """Generate TOTP passcode.
 
     :param bytes secret: A base32 encoded secret for the TOTP authentication
     :returns: totp passcode as bytes
     """
-    if isinstance(secret, six.text_type):
+    if isinstance(secret, str):
         # NOTE(dstanek): since this may be coming from the JSON stored in the
         # database it may be UTF-8 encoded
         secret = secret.encode('utf-8')
@@ -60,28 +67,42 @@ def _generate_totp_passcode(secret):
         secret = secret + b'='
 
     decoded = base64.b32decode(secret)
+    # NOTE(lhinds) This is marked as #nosec since bandit will see SHA1
+    # which is marked as insecure. In this instance however, keystone uses
+    # HMAC-SHA1 when generating the TOTP, which is currently not insecure but
+    # will still trigger when scanned by bandit.
     totp = crypto_totp.TOTP(
-        decoded, 6, hashes.SHA1(), 30, backend=default_backend())
-    return totp.generate(timeutils.utcnow_ts(microsecond=True)).decode('utf-8')
+        decoded, PASSCODE_LENGTH, hashes.SHA1(), PASSCODE_TIME_PERIOD,  # nosec
+        backend=default_backend())
+
+    passcode_ts = timeutils.utcnow_ts(microsecond=True)
+    passcodes = [totp.generate(passcode_ts).decode('utf-8')]
+
+    for i in range(included_previous_windows):
+        # NOTE(adriant): we move back the timestamp the number of seconds in
+        # PASSCODE_TIME_PERIOD each time.
+        passcode_ts -= PASSCODE_TIME_PERIOD
+        passcodes.append(totp.generate(passcode_ts).decode('utf-8'))
+    return passcodes
 
 
-@dependency.requires('credential_api')
 class TOTP(base.AuthMethodHandler):
 
-    def authenticate(self, request, auth_payload, auth_context):
+    def authenticate(self, auth_payload):
         """Try to authenticate using TOTP."""
+        response_data = {}
         user_info = plugins.TOTPUserInfo.create(auth_payload, METHOD_NAME)
         auth_passcode = auth_payload.get('user').get('passcode')
 
-        credentials = self.credential_api.list_credentials_for_user(
+        credentials = PROVIDERS.credential_api.list_credentials_for_user(
             user_info.user_id, type='totp')
 
         valid_passcode = False
         for credential in credentials:
             try:
-                generated_passcode = _generate_totp_passcode(
-                    credential['blob'])
-                if auth_passcode == generated_passcode:
+                generated_passcodes = _generate_totp_passcodes(
+                    credential['blob'], CONF.totp.included_previous_windows)
+                if auth_passcode in generated_passcodes:
                     valid_passcode = True
                     break
             except (ValueError, KeyError):
@@ -96,4 +117,7 @@ class TOTP(base.AuthMethodHandler):
             msg = _('Invalid username or TOTP passcode')
             raise exception.Unauthorized(msg)
 
-        auth_context['user_id'] = user_info.user_id
+        response_data['user_id'] = user_info.user_id
+
+        return base.AuthHandlerResponse(status=True, response_body=None,
+                                        response_data=response_data)
