@@ -15,27 +15,36 @@ from oslo_policy import policy
 
 from keystone.common.policies import base
 
-deprecated_ec2_get_credential = policy.DeprecatedRule(
-    name=base.IDENTITY % 'ec2_get_credential',
-    check_str=base.RULE_ADMIN_OR_CREDENTIAL_OWNER
-)
-deprecated_ec2_list_credentials = policy.DeprecatedRule(
-    name=base.IDENTITY % 'ec2_list_credentials',
-    check_str=base.RULE_ADMIN_OR_OWNER
-)
-deprecated_ec2_create_credential = policy.DeprecatedRule(
-    name=base.IDENTITY % 'ec2_create_credential',
-    check_str=base.RULE_ADMIN_OR_OWNER
-)
-deprecated_ec2_delete_credential = policy.DeprecatedRule(
-    name=base.IDENTITY % 'ec2_delete_credential',
-    check_str=base.RULE_ADMIN_OR_CREDENTIAL_OWNER
-)
-
 DEPRECATED_REASON = (
     "The EC2 credential API is now aware of system scope and default roles."
 )
 
+deprecated_ec2_get_credential = policy.DeprecatedRule(
+    name=base.IDENTITY % 'ec2_get_credential',
+    check_str=base.RULE_ADMIN_OR_CREDENTIAL_OWNER,
+    deprecated_reason=DEPRECATED_REASON,
+    deprecated_since=versionutils.deprecated.TRAIN
+)
+deprecated_ec2_list_credentials = policy.DeprecatedRule(
+    name=base.IDENTITY % 'ec2_list_credentials',
+    check_str=base.RULE_ADMIN_OR_OWNER,
+    deprecated_reason=DEPRECATED_REASON,
+    deprecated_since=versionutils.deprecated.TRAIN
+)
+deprecated_ec2_create_credential = policy.DeprecatedRule(
+    name=base.IDENTITY % 'ec2_create_credential',
+    check_str=base.RULE_ADMIN_OR_OWNER,
+    deprecated_reason=DEPRECATED_REASON,
+    deprecated_since=versionutils.deprecated.TRAIN
+)
+deprecated_ec2_delete_credential = policy.DeprecatedRule(
+    name=base.IDENTITY % 'ec2_delete_credential',
+    check_str=base.RULE_ADMIN_OR_CREDENTIAL_OWNER,
+    deprecated_reason=DEPRECATED_REASON,
+    deprecated_since=versionutils.deprecated.TRAIN
+)
+
+
 ec2_credential_policies = [
     policy.DocumentedRuleDefault(
         name=base.IDENTITY % 'ec2_get_credential',
@@ -45,9 +54,7 @@ ec2_credential_policies = [
         operations=[{'path': ('/v3/users/{user_id}/credentials/OS-EC2/'
                               '{credential_id}'),
                      'method': 'GET'}],
-        deprecated_rule=deprecated_ec2_get_credential,
-        deprecated_reason=DEPRECATED_REASON,
-        deprecated_since=versionutils.deprecated.TRAIN
+        deprecated_rule=deprecated_ec2_get_credential
     ),
     policy.DocumentedRuleDefault(
         name=base.IDENTITY % 'ec2_list_credentials',
@@ -57,8 +64,6 @@ ec2_credential_policies = [
         operations=[{'path': '/v3/users/{user_id}/credentials/OS-EC2',
                      'method': 'GET'}],
         deprecated_rule=deprecated_ec2_list_credentials,
-        deprecated_reason=DEPRECATED_REASON,
-        deprecated_since=versionutils.deprecated.TRAIN
     ),
     policy.DocumentedRuleDefault(
         name=base.IDENTITY % 'ec2_create_credential',
@@ -68,8 +73,6 @@ ec2_credential_policies = [
         operations=[{'path': '/v3/users/{user_id}/credentials/OS-EC2',
                      'method': 'POST'}],
         deprecated_rule=deprecated_ec2_create_credential,
-        deprecated_reason=DEPRECATED_REASON,
-        deprecated_since=versionutils.deprecated.TRAIN
     ),
     policy.DocumentedRuleDefault(
         name=base.IDENTITY % 'ec2_delete_credential',
@@ -80,8 +83,6 @@ ec2_credential_policies = [
                               '{credential_id}'),
                      'method': 'DELETE'}],
         deprecated_rule=deprecated_ec2_delete_credential,
-        deprecated_reason=DEPRECATED_REASON,
-        deprecated_since=versionutils.deprecated.TRAIN
     )
 ]
 
