@@ -35,12 +35,7 @@ def get_enforcer():
 
     :returns: :class:`oslo_policy.policy.Enforcer`
     """
-    # Here we pass an empty list of arguments because there aren't any
-    # arguments that oslo.config or oslo.policy shouldn't already understand
-    # from the CONF object. This makes things easier here because we don't have
-    # to parse arguments passed in from the command line and remove unexpected
-    # arguments before building a Config object.
-    CONF([], project='keystone')
+    CONF(project='keystone')
     return _ENFORCER._enforcer
 
 
