@@ -16,21 +16,20 @@
 
 import json
 
-from oslo_log import versionutils
-
-from keystone.common import dependency
+from keystone.common import cache
 from keystone.common import driver_hints
 from keystone.common import manager
+from keystone.common import provider_api
 import keystone.conf
-from keystone.credential.backends import base
 from keystone import exception
+from keystone import notifications
 
 
 CONF = keystone.conf.CONF
+MEMOIZE = cache.get_memoization_decorator(group='credential')
+PROVIDERS = provider_api.ProviderAPIs
 
 
-@dependency.provider('credential_api')
-@dependency.requires('credential_provider_api')
 class Manager(manager.Manager):
     """Default pivot point for the Credential backend.
 
@@ -40,6 +39,9 @@ class Manager(manager.Manager):
     """
 
     driver_namespace = 'keystone.credential'
+    _provides_api = 'credential_api'
+
+    _CRED = 'credential'
 
     def __init__(self):
         super(Manager, self).__init__(CONF.credential.driver)
@@ -48,12 +50,12 @@ class Manager(manager.Manager):
         """Return a decrypted credential reference."""
         if credential['type'] == 'ec2':
             decrypted_blob = json.loads(
-                self.credential_provider_api.decrypt(
+                PROVIDERS.credential_provider_api.decrypt(
                     credential['encrypted_blob'],
                 )
             )
         else:
-            decrypted_blob = self.credential_provider_api.decrypt(
+            decrypted_blob = PROVIDERS.credential_provider_api.decrypt(
                 credential['encrypted_blob']
             )
         credential['blob'] = decrypted_blob
@@ -68,18 +70,30 @@ class Manager(manager.Manager):
             # NOTE(lbragstad): When dealing with ec2 credentials, it's possible
             # for the `blob` to be a dictionary. Let's make sure we are
             # encrypting a string otherwise encryption will fail.
-            encrypted_blob, key_hash = self.credential_provider_api.encrypt(
-                json.dumps(credential['blob'])
+            encrypted_blob, key_hash = (
+                PROVIDERS.credential_provider_api.encrypt(
+                    json.dumps(credential['blob'])
+                )
             )
         else:
-            encrypted_blob, key_hash = self.credential_provider_api.encrypt(
-                credential['blob']
+            encrypted_blob, key_hash = (
+                PROVIDERS.credential_provider_api.encrypt(
+                    credential['blob']
+                )
             )
         credential_copy['encrypted_blob'] = encrypted_blob
         credential_copy['key_hash'] = key_hash
         credential_copy.pop('blob', None)
         return credential_copy
 
+    def _assert_limit_not_exceeded(self, user_id):
+        user_limit = CONF.credential.user_limit
+        if user_limit >= 0:
+            cred_count = len(self.list_credentials_for_user(user_id))
+            if cred_count >= user_limit:
+                raise exception.CredentialLimitExceeded(
+                    limit=user_limit)
+
     @manager.response_truncated
     def list_credentials(self, hints=None):
         credentials = self.driver.list_credentials(
@@ -90,24 +104,49 @@ class Manager(manager.Manager):
         return credentials
 
     def list_credentials_for_user(self, user_id, type=None):
-        """List credentials for a specific user."""
-        credentials = self.driver.list_credentials_for_user(user_id, type=type)
+        credentials = self._list_credentials_for_user(user_id, type)
         for credential in credentials:
             credential = self._decrypt_credential(credential)
         return credentials
 
+    @MEMOIZE
+    def _list_credentials_for_user(self, user_id, type):
+        """List credentials for a specific user."""
+        return self.driver.list_credentials_for_user(user_id, type)
+
     def get_credential(self, credential_id):
         """Return a credential reference."""
-        credential = self.driver.get_credential(credential_id)
+        credential = self._get_credential(credential_id)
         return self._decrypt_credential(credential)
 
-    def create_credential(self, credential_id, credential):
+    @MEMOIZE
+    def _get_credential(self, credential_id):
+        return self.driver.get_credential(credential_id)
+
+    def create_credential(self, credential_id, credential,
+                          initiator=None):
         """Create a credential."""
         credential_copy = self._encrypt_credential(credential)
+        user_id = credential_copy['user_id']
+        self._assert_limit_not_exceeded(user_id)
         ref = self.driver.create_credential(credential_id, credential_copy)
+        if MEMOIZE.should_cache(ref):
+            self._get_credential.set(ref,
+                                     credential_copy,
+                                     credential_id)
+            self._list_credentials_for_user.invalidate(self,
+                                                       ref['user_id'],
+                                                       ref['type'])
+            self._list_credentials_for_user.invalidate(self,
+                                                       ref['user_id'],
+                                                       None)
         ref.pop('key_hash', None)
         ref.pop('encrypted_blob', None)
         ref['blob'] = credential['blob']
+        notifications.Audit.created(
+            self._CRED,
+            credential_id,
+            initiator)
         return ref
 
     def _validate_credential_update(self, credential_id, credential):
@@ -132,6 +171,14 @@ class Manager(manager.Manager):
             existing_credential = self.get_credential(credential_id)
             existing_blob = existing_credential['blob']
         ref = self.driver.update_credential(credential_id, credential_copy)
+        if MEMOIZE.should_cache(ref):
+            self._get_credential.set(ref, self, credential_id)
+            self._list_credentials_for_user.invalidate(self,
+                                                       ref['user_id'],
+                                                       ref['type'])
+            self._list_credentials_for_user.invalidate(self,
+                                                       ref['user_id'],
+                                                       None)
         ref.pop('key_hash', None)
         ref.pop('encrypted_blob', None)
         # If the update request contains a `blob` attribute - we should return
@@ -143,14 +190,46 @@ class Manager(manager.Manager):
             ref['blob'] = existing_blob
         return ref
 
+    def delete_credential(self, credential_id,
+                          initiator=None):
+        """Delete a credential."""
+        cred = self.get_credential(credential_id)
+        self.driver.delete_credential(credential_id)
+        self._get_credential.invalidate(self, credential_id)
+        self._list_credentials_for_user.invalidate(self,
+                                                   cred['user_id'],
+                                                   cred['type'])
+        self._list_credentials_for_user.invalidate(self,
+                                                   cred['user_id'],
+                                                   None)
+        notifications.Audit.deleted(
+            self._CRED, credential_id, initiator)
 
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.credential.CredentialDriverV8',
-    in_favor_of='keystone.credential.backends.base.CredentialDriverV8',
-    remove_in=+1)
-class AuthMethodHandler(base.CredentialDriverV8):
-    pass
+    def delete_credentials_for_project(self, project_id):
+        """Delete all credentials for a project."""
+        hints = driver_hints.Hints()
+        hints.add_filter('project_id', project_id)
+        creds = self.driver.list_credentials(hints)
 
+        self.driver.delete_credentials_for_project(project_id)
+        for cred in creds:
+            self._get_credential.invalidate(self, cred['id'])
+            self._list_credentials_for_user.invalidate(self,
+                                                       cred['user_id'],
+                                                       cred['type'])
+            self._list_credentials_for_user.invalidate(self,
+                                                       cred['user_id'],
+                                                       None)
 
-Driver = manager.create_legacy_driver(base.CredentialDriverV8)
+    def delete_credentials_for_user(self, user_id):
+        """Delete all credentials for a user."""
+        creds = self.driver.list_credentials_for_user(user_id)
+        self.driver.delete_credentials_for_user(user_id)
+        for cred in creds:
+            self._get_credential.invalidate(self, cred['id'])
+            self._list_credentials_for_user.invalidate(self,
+                                                       user_id,
+                                                       cred['type'])
+            self._list_credentials_for_user.invalidate(self,
+                                                       cred['user_id'],
+                                                       None)
