@@ -15,6 +15,7 @@
 import abc
 import codecs
 import os.path
+import random
 import re
 import sys
 import uuid
@@ -860,11 +861,22 @@ class PooledLDAPHandler(LDAPHandler):
         cleaned up when message.clean() is called.
 
         """
-        results = message.connection.result3(message.id, all, timeout)
-
-        # Now that we have the results from the LDAP server for the message, we
-        # don't need the the context manager used to create the connection.
-        message.clean()
+        # message.connection.result3 might throw an exception
+        # so the code must ensure that message.clean() is invoked
+        # regardless of the result3's result. Otherwise, the
+        # connection will be marked as active forever, which
+        # ultimately renders the pool unusable, causing a DoS.
+        try:
+            results = message.connection.result3(message.id, all, timeout)
+        except Exception:
+            # We don't want to ignore thrown
+            # exceptions, raise them
+            raise
+        finally:
+            # Now that we have the results from the LDAP server for
+            # the message, we don't need the the context manager used
+            # to create the connection.
+            message.clean()
 
         return results
 
@@ -1155,7 +1167,12 @@ class BaseLdap(object):
     tree_dn = None
 
     def __init__(self, conf):
-        self.LDAP_URL = conf.ldap.url
+        if conf.ldap.randomize_urls:
+            urls = re.split(r'[\s,]+', conf.ldap.url)
+            random.shuffle(urls)
+            self.LDAP_URL = ','.join(urls)
+        else:
+            self.LDAP_URL = conf.ldap.url
         self.LDAP_USER = conf.ldap.user
         self.LDAP_PASSWORD = conf.ldap.password
         self.LDAP_SCOPE = ldap_scope(conf.ldap.query_scope)
