@@ -13,21 +13,19 @@
 """Main entry point into the Resource service."""
 
 from oslo_log import log
-import six
 
 from keystone import assignment
 from keystone.common import cache
-from keystone.common import clean
 from keystone.common import driver_hints
 from keystone.common import manager
 from keystone.common import provider_api
+from keystone.common.resource_options import options as ro_opt
 from keystone.common import utils
 import keystone.conf
 from keystone import exception
 from keystone.i18n import _
 from keystone import notifications
 from keystone.resource.backends import base
-from keystone.resource.backends import sql as resource_sql
 from keystone.token import provider as token_provider
 
 CONF = keystone.conf.CONF
@@ -55,14 +53,8 @@ class Manager(manager.Manager):
     _PROJECT_TAG = 'project tag'
 
     def __init__(self):
-        # NOTE(morgan): The resource driver must be SQL. This is because there
-        # is a FK between identity and resource. Almost every deployment uses
-        # SQL Identity in some form. Even if SQL Identity is not used, there
-        # is almost no reason to have non-SQL Resource. Keystone requires
-        # SQL in a number of ways, this simply codifies it plainly for resource
-        # the driver_name = None simply implies we don't need to load a driver.
-        self.driver = resource_sql.Resource()
-        super(Manager, self).__init__(driver_name=None)
+        resource_driver = CONF.resource.driver
+        super(Manager, self).__init__(resource_driver)
 
     def _get_hierarchy_depth(self, parents_list):
         return len(parents_list) + 1
@@ -102,7 +94,8 @@ class Manager(manager.Manager):
             was not satisfied.
         """
         if (not PROVIDERS.identity_api.multiple_domains_supported and
-                project_ref['id'] != CONF.identity.default_domain_id):
+                project_ref['id'] != CONF.identity.default_domain_id and
+                project_ref['id'] != base.NULL_DOMAIN_ID):
             raise exception.ValidationError(
                 message=_('Multiple domains are not supported'))
 
@@ -186,8 +179,10 @@ class Manager(manager.Manager):
                      ) % project['name']
         else:
             return _('it is not permitted to have two projects '
-                     'with the same name in the same domain : %s'
-                     ) % project['name']
+                     'with either the same name or same id in '
+                     'the same domain: '
+                     'name is %(name)s, project id %(id)s'
+                     ) % project
 
     def create_project(self, project_id, project, initiator=None):
         project = project.copy()
@@ -290,6 +285,10 @@ class Manager(manager.Manager):
                     _('Cannot enable project %s since it has disabled '
                       'parents') % project_id)
 
+    def _is_immutable(self, project_ref):
+        return project_ref['options'].get(
+            ro_opt.IMMUTABLE_OPT.option_name, False)
+
     def _check_whole_subtree_is_disabled(self, project_id, subtree_list=None):
         if not subtree_list:
             subtree_list = self.list_projects_in_subtree(project_id)
@@ -304,13 +303,23 @@ class Manager(manager.Manager):
         self._require_matching_domain_id(project, original_project)
 
         if original_project['is_domain']:
+            # prevent updates to immutable domains
+            ro_opt.check_immutable_update(
+                original_resource_ref=original_project,
+                new_resource_ref=project,
+                type='domain',
+                resource_id=project_id)
             domain = self._get_domain_from_project(original_project)
             self.assert_domain_not_federated(project_id, domain)
-            if 'enabled' in domain:
-                domain['enabled'] = clean.domain_enabled(domain['enabled'])
             url_safe_option = CONF.resource.domain_name_url_safe
             exception_entity = 'Domain'
         else:
+            # prevent updates to immutable projects
+            ro_opt.check_immutable_update(
+                original_resource_ref=original_project,
+                new_resource_ref=project,
+                type='project',
+                resource_id=project_id)
             url_safe_option = CONF.resource.project_name_url_safe
             exception_entity = 'Project'
 
@@ -321,8 +330,7 @@ class Manager(manager.Manager):
             self._raise_reserved_character_exception(exception_entity,
                                                      project['name'])
         elif project_name_changed:
-            project['name'] = clean.project_name(project['name'])
-
+            project['name'] = project['name'].strip()
         parent_id = original_project.get('parent_id')
         if 'parent_id' in project and project.get('parent_id') != parent_id:
             raise exception.ForbiddenNotSecurity(
@@ -469,13 +477,17 @@ class Manager(manager.Manager):
         """
         project = self.driver.get_project(project_id)
         if project.get('is_domain'):
-            self.delete_domain(project_id, initiator)
+            self._delete_domain(project, initiator)
         else:
-            self._delete_project(project_id, initiator, cascade)
+            self._delete_project(project, initiator, cascade)
 
-    def _delete_project(self, project_id, initiator=None, cascade=False):
-        # Use the driver directly to prevent using old cached value.
-        project = self.driver.get_project(project_id)
+    def _delete_project(self, project, initiator=None, cascade=False):
+        # Prevent deletion of immutable projects
+        ro_opt.check_immutable_delete(
+            resource_ref=project,
+            resource_type='project',
+            resource_id=project['id'])
+        project_id = project['id']
         if project['is_domain'] and project['enabled']:
             raise exception.ValidationError(
                 message=_('cannot delete an enabled project acting as a '
@@ -787,7 +799,14 @@ class Manager(manager.Manager):
             domain = self.driver.get_project(domain_id)
         except exception.ProjectNotFound:
             raise exception.DomainNotFound(domain_id=domain_id)
+        self._delete_domain(domain, initiator)
 
+    def _delete_domain(self, domain, initiator=None):
+        # Disallow deletion of immutable domains
+        ro_opt.check_immutable_delete(
+            resource_ref=domain,
+            resource_type='domain',
+            resource_id=domain['id'])
         # To help avoid inadvertent deletes, we insist that the domain
         # has been previously disabled.  This also prevents a user deleting
         # their own domain since, once it is disabled, they won't be able
@@ -797,11 +816,12 @@ class Manager(manager.Manager):
                 _('Cannot delete a domain that is enabled, please disable it '
                   'first.'))
 
+        domain_id = domain['id']
         self._delete_domain_contents(domain_id)
         notifications.Audit.internal(
             notifications.DOMAIN_DELETED, domain_id
         )
-        self._delete_project(domain_id, initiator)
+        self._delete_project(domain, initiator)
         try:
             self.get_domain.invalidate(self, domain_id)
             self.get_domain_by_name.invalidate(self, domain['name'])
@@ -836,7 +856,7 @@ class Manager(manager.Manager):
                 _delete_projects(proj, projects, examined)
 
             try:
-                self.delete_project(project['id'], initiator=None)
+                self._delete_project(project, initiator=None)
             except exception.ProjectNotFound:
                 LOG.debug(('Project %(projectid)s not found when '
                            'deleting domain contents for %(domainid)s, '
@@ -911,6 +931,12 @@ class Manager(manager.Manager):
         :returns: The value of the created tag
         """
         project = self.driver.get_project(project_id)
+        if ro_opt.check_resource_immutable(resource_ref=project):
+            raise exception.ResourceUpdateForbidden(
+                message=_(
+                    'Cannot create project tags for %(project_id)s, project '
+                    'is immutable. Set "immutable" option to false before '
+                    'creating project tags.') % {'project_id': project_id})
         tag_name = tag.strip()
         project['tags'].append(tag_name)
         self.update_project(project_id, {'tags': project['tags']})
@@ -952,7 +978,13 @@ class Manager(manager.Manager):
 
         :returns: A list of tags
         """
-        self.driver.get_project(project_id)
+        project = self.driver.get_project(project_id)
+        if ro_opt.check_resource_immutable(resource_ref=project):
+            raise exception.ResourceUpdateForbidden(
+                message=_(
+                    'Cannot update project tags for %(project_id)s, project '
+                    'is immutable. Set "immutable" option to false before '
+                    'creating project tags.') % {'project_id': project_id})
         tag_list = [t.strip() for t in tags]
         project = {'tags': tag_list}
         self.update_project(project_id, project)
@@ -968,6 +1000,12 @@ class Manager(manager.Manager):
             does not exist on the project
         """
         project = self.driver.get_project(project_id)
+        if ro_opt.check_resource_immutable(resource_ref=project):
+            raise exception.ResourceUpdateForbidden(
+                message=_(
+                    'Cannot delete project tags for %(project_id)s, project '
+                    'is immutable. Set "immutable" option to false before '
+                    'creating project tags.') % {'project_id': project_id})
         try:
             project['tags'].remove(tag)
         except ValueError:
@@ -1437,7 +1475,7 @@ class DomainConfigManager(manager.Manager):
         sensitive_dict = {s['option']: s['value'] for s in sensitive}
 
         for each_whitelisted in whitelisted:
-            if not isinstance(each_whitelisted['value'], six.string_types):
+            if not isinstance(each_whitelisted['value'], str):
                 # We only support substitutions into string types, if its an
                 # integer, list etc. then just continue onto the next one
                 continue
