@@ -13,22 +13,22 @@
 import collections
 import os
 
-import oslo_i18n
 from oslo_log import log
 import stevedore
-from werkzeug.contrib import fixers
 
-
-# NOTE(dstanek): i18n.enable_lazy() must be called before
-# keystone.i18n._() is called to ensure it has the desired lazy lookup
-# behavior. This includes cases, like keystone.exceptions, where
-# keystone.i18n._() is called at import time.
-oslo_i18n.enable_lazy()
+try:
+    # werkzeug 0.15.x
+    from werkzeug.middleware import proxy_fix
+except ImportError:
+    # werkzeug 0.14.x
+    from werkzeug.contrib import fixers as proxy_fix
 
 from keystone.common import profiler
 import keystone.conf
 import keystone.server
 from keystone.server.flask import application
+from keystone.server.flask.request_processing.middleware import auth_context
+from keystone.server.flask.request_processing.middleware import url_normalize
 
 # NOTE(morgan): Middleware Named Tuple with the following values:
 #   * "namespace": namespace for the entry_point
@@ -51,21 +51,23 @@ _APP_MIDDLEWARE = (
     _Middleware(namespace='keystone.server_middleware',
                 ep='http_proxy_to_wsgi',
                 conf={}),
-    _Middleware(namespace='keystone.server_middleware',
-                ep='url_normalize',
-                conf={}),
-    _Middleware(namespace='keystone.server_middleware',
-                ep='json_body',
-                conf={}),
     _Middleware(namespace='keystone.server_middleware',
                 ep='osprofiler',
                 conf={}),
     _Middleware(namespace='keystone.server_middleware',
                 ep='request_id',
                 conf={}),
-    _Middleware(namespace='keystone.server_middleware',
-                ep='build_auth_context',
-                conf={}),
+)
+
+# NOTE(morgan): ORDER HERE IS IMPORTANT! Each of these middlewares are
+# implemented/defined explicitly in Keystone Server. They do some level of
+# lifting to ensure the request is properly handled. It is importat to note
+# that these will be processed in the order of this list AND after all
+# middleware defined in _APP_MIDDLEWARE. AuthContextMiddleware should always
+# be the last element here as long as it is an actual Middleware.
+_KEYSTONE_MIDDLEWARE = (
+    url_normalize.URLNormalizingMiddleware,
+    auth_context.AuthContextMiddleware,
 )
 
 
@@ -92,6 +94,7 @@ def setup_app_middleware(app):
     # processes the request first.
 
     MW = _APP_MIDDLEWARE
+    IMW = _KEYSTONE_MIDDLEWARE
 
     # Add in optional (config-based) middleware
     # NOTE(morgan): Each of these may need to be in a specific location
@@ -102,6 +105,11 @@ def setup_app_middleware(app):
                           ep='debug',
                           conf={}),) + _APP_MIDDLEWARE
 
+    # Apply internal-only Middleware (e.g. AuthContextMiddleware). These
+    # are below all externally loaded middleware in request processing.
+    for mw in reversed(IMW):
+        app.wsgi_app = mw(app.wsgi_app)
+
     # Apply the middleware to the application.
     for mw in reversed(MW):
         # TODO(morgan): Explore moving this to ExtensionManager, but we
@@ -124,8 +132,8 @@ def setup_app_middleware(app):
         factory_func = loaded.driver.factory({}, **mw.conf)
         app.wsgi_app = factory_func(app.wsgi_app)
 
-    # Apply werkzeug speficic middleware
-    app.wsgi_app = fixers.ProxyFix(app.wsgi_app)
+    # Apply werkzeug specific middleware
+    app.wsgi_app = proxy_fix.ProxyFix(app.wsgi_app)
     return app
 
 
