@@ -11,6 +11,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+
 import atexit
 import base64
 import contextlib
@@ -18,13 +19,14 @@ import datetime
 import functools
 import hashlib
 import json
+import secrets
+
 import ldap
 import os
 import shutil
 import socket
 import sys
 import uuid
-import warnings
 
 import fixtures
 import flask
@@ -36,7 +38,6 @@ from oslo_context import fixture as oslo_ctx_fixture
 from oslo_log import fixture as log_fixture
 from oslo_log import log
 from oslo_utils import timeutils
-from sqlalchemy import exc
 import testtools
 from testtools import testcase
 
@@ -422,9 +423,9 @@ def new_ec2_credential(user_id, project_id=None, blob=None, **kwargs):
 
 def new_totp_credential(user_id, project_id=None, blob=None):
     if not blob:
-        # NOTE(notmorgan): 20 bytes of data from os.urandom for
+        # NOTE(notmorgan): 20 bytes of data from secrets.token_bytes for
         # a totp secret.
-        blob = base64.b32encode(os.urandom(20)).decode('utf-8')
+        blob = base64.b32encode(secrets.token_bytes(20)).decode('utf-8')
     credential = new_credential_ref(user_id=user_id,
                                     project_id=project_id,
                                     blob=blob,
@@ -678,14 +679,9 @@ class BaseTestCase(testtools.TestCase):
         self.useFixture(fixtures.MockPatchObject(sys, 'exit',
                                                  side_effect=UnexpectedExit))
         self.useFixture(log_fixture.get_logging_handle_error_fixture())
+        self.stdlog = self.useFixture(ksfixtures.StandardLogging())
+        self.useFixture(ksfixtures.WarningsFixture())
 
-        warnings.filterwarnings('error', category=DeprecationWarning,
-                                module='^keystone\\.')
-        warnings.simplefilter('error', exc.SAWarning)
-        if hasattr(exc, "RemovedIn20Warning"):
-            warnings.simplefilter('ignore', exc.RemovedIn20Warning)
-
-        self.addCleanup(warnings.resetwarnings)
         # Ensure we have an empty threadlocal context at the start of each
         # test.
         self.assertIsNone(oslo_context.get_current())
@@ -1020,8 +1016,8 @@ class TestCase(BaseTestCase):
             value,
             "%s != %s" % (expected, value))
 
-    def assertNotEmpty(self, l):
-        self.assertGreater(len(l), 0)
+    def assertNotEmpty(self, iterable):
+        self.assertGreater(len(iterable), 0)
 
     def assertUserDictEqual(self, expected, observed, message=''):
         """Assert that a user dict is equal to another user dict.
