@@ -28,8 +28,6 @@ import shelve
 
 import ldap
 from oslo_log import log
-import six
-from six import moves
 
 import keystone.conf
 from keystone import exception
@@ -119,7 +117,7 @@ def _paren_groups(source):
     count = 0
     start = 0
     result = []
-    for pos in moves.range(len(source)):
+    for pos in range(len(source)):
         if source[pos] == '(':
             if count == 0:
                 start = pos
@@ -168,8 +166,8 @@ def _match(key, value, attrs):
         # For serviceId, the backend is returning a list of numbers.
         # Make sure we convert them to strings first before comparing
         # them.
-        str_sids = [six.text_type(x) for x in attrs[key]]
-        return six.text_type(value) in str_sids
+        str_sids = [str(x) for x in attrs[key]]
+        return str(value) in str_sids
     if key != 'objectclass':
         check_value = _internal_attr(key, value)[0].lower()
         norm_values = list(
@@ -191,9 +189,9 @@ def _subs(value):
     so subclasses need to be defined manually in the dictionary below.
 
     """
-    subs = {'groupOfNames': ['keystoneTenant',
+    subs = {'groupOfNames': ['keystoneProject',
                              'keystoneRole',
-                             'keystoneTenantRole']}
+                             'keystoneProjectRole']}
     if value in subs:
         return [value] + subs[value]
     return [value]
@@ -298,6 +296,9 @@ class FakeLdap(common.LDAPHandler):
             raise ldap.SERVER_DOWN
         whos = ['cn=Admin', CONF.ldap.user]
         if (who in whos and cred in ['password', CONF.ldap.password]):
+            self.connected = True
+            self.who = who
+            self.cred = cred
             return
 
         attrs = self.db.get(self.key(who))
@@ -318,6 +319,9 @@ class FakeLdap(common.LDAPHandler):
 
     def unbind_s(self):
         """Provide for compatibility but this method is ignored."""
+        self.connected = False
+        self.who = None
+        self.cred = None
         if server_fail:
             raise ldap.SERVER_DOWN
 
@@ -536,7 +540,7 @@ class FakeLdap(common.LDAPHandler):
             raise exception.NotImplemented()
 
         # only passing a single server control is supported by this fake ldap
-        if len(serverctrls) > 1:
+        if serverctrls and len(serverctrls) > 1:
             raise exception.NotImplemented()
 
         # search_ext is async and returns an identifier used for
@@ -591,6 +595,7 @@ class FakeLdapPool(FakeLdap):
     def __init__(self, uri, retry_max=None, retry_delay=None, conn=None):
         super(FakeLdapPool, self).__init__(conn=conn)
         self.url = uri
+        self._uri = uri
         self.connected = None
         self.conn = self
         self._connection_time = 5  # any number greater than 0
