@@ -16,7 +16,7 @@ from keystone.common import driver_hints
 from keystone import exception
 
 
-class IdentityDriverV8Tests(object):
+class IdentityDriverTests(object):
     driver = None  # subclasses must override driver to the actual driver.
 
     # subclasses that don't allow name updates must set this to False.
@@ -78,10 +78,6 @@ class IdentityDriverV8Tests(object):
         self.assertIs(self.expected_is_domain_aware,
                       self.driver.is_domain_aware())
 
-    def test_default_assignment_driver(self):
-        self.assertEqual(self.expected_default_assignment_driver,
-                         self.driver.default_assignment_driver())
-
     def test_is_sql(self):
         self.assertIs(self.expected_is_sql, self.driver.is_sql)
 
@@ -111,7 +107,8 @@ class IdentityDriverV8Tests(object):
             'password': uuid.uuid4().hex,
             'enabled': True,
             'default_project_id': uuid.uuid4().hex,
-            'password_expires_at': None
+            'password_expires_at': None,
+            'options': {}
         }
         if self.driver.is_domain_aware():
             user['domain_id'] = uuid.uuid4().hex
