@@ -14,19 +14,22 @@
 
 import itertools
 import os
+from unittest import mock
 import uuid
 
-import mock
 from oslo_config import fixture as config_fixture
 
+from keystone.common import provider_api
 import keystone.conf
 from keystone import exception
 from keystone import identity
 from keystone.tests import unit
+from keystone.tests.unit import default_fixtures
 from keystone.tests.unit.ksfixtures import database
 
 
 CONF = keystone.conf.CONF
+PROVIDERS = provider_api.ProviderAPIs
 
 
 class TestDomainConfigs(unit.BaseTestCase):
@@ -102,8 +105,8 @@ class TestDomainConfigs(unit.BaseTestCase):
                 name,
                 identity.DOMAIN_CONF_FTAIL)))
 
-        walk_fake = lambda *a, **kwa: (
-            ('/fake/keystone/domains/config', [], files), )
+        def walk_fake(*a, **kwa):
+            return ('/fake/keystone/domains/config', [], files),
 
         generic_driver = mock.Mock(is_sql=False)
 
@@ -131,6 +134,8 @@ class TestDatabaseDomainConfigs(unit.TestCase):
         super(TestDatabaseDomainConfigs, self).setUp()
         self.useFixture(database.Database())
         self.load_backends()
+        PROVIDERS.resource_api.create_domain(
+            default_fixtures.ROOT_DOMAIN['id'], default_fixtures.ROOT_DOMAIN)
 
     def test_domain_config_in_database_disabled_by_default(self):
         self.assertFalse(CONF.identity.domain_configurations_from_database)
@@ -139,18 +144,18 @@ class TestDatabaseDomainConfigs(unit.TestCase):
         self.config_fixture.config(domain_configurations_from_database=True,
                                    group='identity')
         domain = unit.new_domain_ref()
-        self.resource_api.create_domain(domain['id'], domain)
+        PROVIDERS.resource_api.create_domain(domain['id'], domain)
         # Override two config options for our domain
         conf = {'ldap': {'url': uuid.uuid4().hex,
                          'suffix': uuid.uuid4().hex,
-                         'use_tls': 'True'},
+                         'use_tls': True},
                 'identity': {
                     'driver': 'ldap'}}
-        self.domain_config_api.create_config(domain['id'], conf)
+        PROVIDERS.domain_config_api.create_config(domain['id'], conf)
         fake_standard_driver = None
         domain_config = identity.DomainConfigs()
         domain_config.setup_domain_drivers(fake_standard_driver,
-                                           self.resource_api)
+                                           PROVIDERS.resource_api)
         # Make sure our two overrides are in place, and others are not affected
         res = domain_config.get_domain_conf(domain['id'])
         self.assertEqual(conf['ldap']['url'], res.ldap.url)
@@ -170,58 +175,9 @@ class TestDatabaseDomainConfigs(unit.TestCase):
 
         domain_config = identity.DomainConfigs()
         domain_config.setup_domain_drivers(fake_standard_driver,
-                                           self.resource_api)
+                                           PROVIDERS.resource_api)
         res = domain_config.get_domain_conf(domain['id'])
         self.assertEqual(CONF.ldap.url, res.ldap.url)
         self.assertEqual(CONF.ldap.suffix, res.ldap.suffix)
         self.assertEqual(CONF.ldap.use_tls, res.ldap.use_tls)
         self.assertEqual(CONF.ldap.query_scope, res.ldap.query_scope)
-
-
-class TestShadowUsers(unit.TestCase):
-
-    def setUp(self):
-        super(TestShadowUsers, self).setUp()
-        self.useFixture(database.Database())
-        self.load_backends()
-
-    def test_shadow_federated_user(self):
-        fed_user = unit.new_federated_user_ref()
-        user = (
-            self.identity_api.shadow_federated_user(fed_user['idp_id'],
-                                                    fed_user['protocol_id'],
-                                                    fed_user['unique_id'],
-                                                    fed_user['display_name'])
-        )
-        self.assertIsNotNone(user['id'])
-        self.assertEqual(5, len(user.keys()))
-        self.assertIsNotNone(user['name'])
-        self.assertIsNone(user['password_expires_at'])
-        self.assertIsNone(user['domain_id'])
-        self.assertEqual(True, user['enabled'])
-
-    def test_shadow_existing_federated_user(self):
-        fed_user = unit.new_federated_user_ref()
-
-        # introduce the user to keystone for the first time
-        shadow_user1 = self.identity_api.shadow_federated_user(
-            fed_user['idp_id'],
-            fed_user['protocol_id'],
-            fed_user['unique_id'],
-            fed_user['display_name'])
-        self.assertEqual(fed_user['display_name'], shadow_user1['name'])
-
-        # shadow the user again, with another name to invalidate the cache
-        # internally, this operation causes request to the driver. It should
-        # not fail.
-        fed_user['display_name'] = uuid.uuid4().hex
-        shadow_user2 = self.identity_api.shadow_federated_user(
-            fed_user['idp_id'],
-            fed_user['protocol_id'],
-            fed_user['unique_id'],
-            fed_user['display_name'])
-        self.assertEqual(fed_user['display_name'], shadow_user2['name'])
-        self.assertNotEqual(shadow_user1['name'], shadow_user2['name'])
-
-        # The shadowed users still share the same unique ID.
-        self.assertEqual(shadow_user1['id'], shadow_user2['id'])
