@@ -65,7 +65,7 @@ class HackingCode(fixtures.Fixture):
         'expected_errors': [
             (7, 10, 'K001'),
             (10, 15, 'K001'),
-            (10, 29, 'K001'),
+            (10, 28, 'K001'),
             (13, 15, 'K001'),
             (16, 15, 'K001'),
             (16, 31, 'K001'),
@@ -87,6 +87,12 @@ class HackingCode(fixtures.Fixture):
         mutable_default_args['expected_errors'][12] = (28, 27, 'K001')
         mutable_default_args['expected_errors'][14] = (32, 11, 'K001')
 
+    # NOTE(gmann): Python version < 3.8.2 (before Ubuntu Focal) returns
+    # the incorrect col_offset for below function defined in above code sample
+    # def f(foo, bad=[], more_bad=[x for x in range(3)]):
+    if sys.version_info < (3, 8, 2):
+        mutable_default_args['expected_errors'][2] = (10, 29, 'K001')
+
     comments_begin_with_space = {
         'code': """
             # This is a good comment
@@ -220,24 +226,28 @@ class HackingTranslations(fixtures.Fixture):
         },
         {
             'code': """
-                # this should not be an error
+                # this should be an error even if it'll be raised later.
                 L = log.getLogger(__name__)
                 msg = _('text')
                 L.warning(msg)
                 raise Exception(msg)
             """,
-            'expected_errors': [],
+            'expected_errors': [
+                (4, 10, 'K005'),
+            ],
         },
         {
             'code': """
                 L = log.getLogger(__name__)
                 def f():
                     msg = _('text')
-                    L2.warning(msg)
+                    L.warning(msg)
                     something = True  # add an extra statement here
                     raise Exception(msg)
             """,
-            'expected_errors': [],
+            'expected_errors': [
+                (4, 14, 'K005'),
+            ],
         },
         {
             'code': """
@@ -262,6 +272,7 @@ class HackingTranslations(fixtures.Fixture):
                 raise Exception(msg)
             """,
             'expected_errors': [
+                (6, 12, 'K005'),
             ],
         },
         {
@@ -304,7 +315,7 @@ class HackingTranslations(fixtures.Fixture):
                 try:
                     something = True
                 except AssertionError as e:
-                    LOG.warning(six.text_type(e))
+                    LOG.warning(e)
                     raise exception.Unauthorized(e)
             """,
             'expected_errors': [],
@@ -320,6 +331,8 @@ class HackingTranslations(fixtures.Fixture):
                     LOG.warning(msg)
                     raise exception.Unauthorized(message=msg)
             """,
-            'expected_errors': [],
+            'expected_errors': [
+                (7, 16, 'K005'),
+            ],
         },
     ]
