@@ -12,21 +12,22 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import copy
 import datetime
 import logging
 import os
+from unittest import mock
 import uuid
 
 import argparse
+import configparser
 import fixtures
 import freezegun
-import mock
+import http.client
 import oslo_config.fixture
-from oslo_db.sqlalchemy import migration
 from oslo_log import log
-from six.moves import configparser
-from six.moves import http_client
-from six.moves import range
+from oslo_serialization import jsonutils
+from oslo_upgradecheck import upgradecheck
 from testtools import matchers
 
 from keystone.cmd import cli
@@ -39,16 +40,20 @@ from keystone.cmd.doctor import ldap
 from keystone.cmd.doctor import security_compliance
 from keystone.cmd.doctor import tokens
 from keystone.cmd.doctor import tokens_fernet
+from keystone.cmd import status
 from keystone.common import provider_api
 from keystone.common.sql import upgrades
 import keystone.conf
+from keystone import exception
 from keystone.i18n import _
 from keystone.identity.mapping_backends import mapping as identity_mapping
 from keystone.tests import unit
 from keystone.tests.unit import default_fixtures
 from keystone.tests.unit.ksfixtures import database
 from keystone.tests.unit.ksfixtures import ldapdb
+from keystone.tests.unit.ksfixtures import policy
 from keystone.tests.unit.ksfixtures import temporaryfile
+from keystone.tests.unit import mapping_fixtures
 
 
 CONF = keystone.conf.CONF
@@ -95,6 +100,7 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
     def setUp(self):
         self.useFixture(database.Database())
         super(CliBootStrapTestCase, self).setUp()
+        self.bootstrap = cli.BootStrap()
 
     def config_files(self):
         self.config_fixture.register_cli_opt(cli.command_opt)
@@ -108,10 +114,16 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
              default_config_files=config_files)
 
     def test_bootstrap(self):
-        bootstrap = cli.BootStrap()
-        self._do_test_bootstrap(bootstrap)
+        self._do_test_bootstrap(self.bootstrap)
 
     def _do_test_bootstrap(self, bootstrap):
+        try:
+            PROVIDERS.resource_api.create_domain(
+                default_fixtures.ROOT_DOMAIN['id'],
+                default_fixtures.ROOT_DOMAIN)
+        except exception.Conflict:
+            pass
+
         bootstrap.do_bootstrap()
         project = PROVIDERS.resource_api.get_project_by_name(
             bootstrap.project_name,
@@ -139,10 +151,10 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
         self.assertEqual(system_roles[0]['id'], admin_role['id'])
         # NOTE(morganfainberg): Pass an empty context, it isn't used by
         # `authenticate` method.
-        PROVIDERS.identity_api.authenticate(
-            self.make_request(),
-            user['id'],
-            bootstrap.password)
+        with self.make_request():
+            PROVIDERS.identity_api.authenticate(
+                user['id'],
+                bootstrap.password)
 
         if bootstrap.region_id:
             region = PROVIDERS.catalog_api.get_region(bootstrap.region_id)
@@ -171,8 +183,7 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
     def test_bootstrap_is_idempotent_when_password_does_not_change(self):
         # NOTE(morganfainberg): Ensure we can run bootstrap with the same
         # configuration multiple times without erroring.
-        bootstrap = cli.BootStrap()
-        self._do_test_bootstrap(bootstrap)
+        self._do_test_bootstrap(self.bootstrap)
         app = self.loadapp()
         v3_password_data = {
             'auth': {
@@ -180,8 +191,8 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
                     "methods": ["password"],
                     "password": {
                         "user": {
-                            "name": bootstrap.username,
-                            "password": bootstrap.password,
+                            "name": self.bootstrap.username,
+                            "password": self.bootstrap.password,
                             "domain": {
                                 "id": CONF.identity.default_domain_id
                             }
@@ -194,7 +205,7 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
             auth_response = c.post('/v3/auth/tokens',
                                    json=v3_password_data)
             token = auth_response.headers['X-Subject-Token']
-        self._do_test_bootstrap(bootstrap)
+        self._do_test_bootstrap(self.bootstrap)
         # build validation request
         with app.test_client() as c:
             # Get a new X-Auth-Token
@@ -206,12 +217,19 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
             c.get('/v3/auth/tokens',
                   headers={'X-Auth-Token': r.headers['X-Subject-Token'],
                            'X-Subject-Token': token})
+        admin_role = PROVIDERS.role_api.get_role(self.bootstrap.role_id)
+        reader_role = PROVIDERS.role_api.get_role(
+            self.bootstrap.reader_role_id)
+        member_role = PROVIDERS.role_api.get_role(
+            self.bootstrap.member_role_id)
+        self.assertEqual(admin_role['options'], {'immutable': True})
+        self.assertEqual(member_role['options'], {'immutable': True})
+        self.assertEqual(reader_role['options'], {'immutable': True})
 
     def test_bootstrap_is_not_idempotent_when_password_does_change(self):
         # NOTE(lbragstad): Ensure bootstrap isn't idempotent when run with
         # different arguments or configuration values.
-        bootstrap = cli.BootStrap()
-        self._do_test_bootstrap(bootstrap)
+        self._do_test_bootstrap(self.bootstrap)
         app = self.loadapp()
         v3_password_data = {
             'auth': {
@@ -219,8 +237,8 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
                     "methods": ["password"],
                     "password": {
                         "user": {
-                            "name": bootstrap.username,
-                            "password": bootstrap.password,
+                            "name": self.bootstrap.username,
+                            "password": self.bootstrap.password,
                             "domain": {
                                 "id": CONF.identity.default_domain_id
                             }
@@ -237,7 +255,7 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
                 token = auth_response.headers['X-Subject-Token']
             new_passwd = uuid.uuid4().hex
             os.environ['OS_BOOTSTRAP_PASSWORD'] = new_passwd
-            self._do_test_bootstrap(bootstrap)
+            self._do_test_bootstrap(self.bootstrap)
             v3_password_data['auth']['identity']['password']['user'][
                 'password'] = new_passwd
             # Move time forward a second to avoid rev. event capturing the new
@@ -257,15 +275,14 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
                 c.get('/v3/auth/tokens',
                       headers={'X-Auth-Token': r.headers['X-Subject-Token'],
                                'X-Subject-Token': token},
-                      expected_status_code=http_client.NOT_FOUND)
+                      expected_status_code=http.client.NOT_FOUND)
 
     def test_bootstrap_recovers_user(self):
-        bootstrap = cli.BootStrap()
-        self._do_test_bootstrap(bootstrap)
+        self._do_test_bootstrap(self.bootstrap)
 
         # Completely lock the user out.
         user_id = PROVIDERS.identity_api.get_user_by_name(
-            bootstrap.username,
+            self.bootstrap.username,
             'default')['id']
         PROVIDERS.identity_api.update_user(
             user_id,
@@ -273,18 +290,61 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
              'password': uuid.uuid4().hex})
 
         # The second bootstrap run will recover the account.
-        self._do_test_bootstrap(bootstrap)
+        self._do_test_bootstrap(self.bootstrap)
 
         # Sanity check that the original password works again.
-        PROVIDERS.identity_api.authenticate(
-            self.make_request(),
-            user_id,
-            bootstrap.password)
+        with self.make_request():
+            PROVIDERS.identity_api.authenticate(
+                user_id,
+                self.bootstrap.password)
+
+    def test_bootstrap_with_explicit_immutable_roles(self):
+        CONF(args=['bootstrap',
+                   '--bootstrap-password', uuid.uuid4().hex,
+                   '--immutable-roles'],
+             project='keystone')
+        self._do_test_bootstrap(self.bootstrap)
+        admin_role = PROVIDERS.role_api.get_role(self.bootstrap.role_id)
+        reader_role = PROVIDERS.role_api.get_role(
+            self.bootstrap.reader_role_id)
+        member_role = PROVIDERS.role_api.get_role(
+            self.bootstrap.member_role_id)
+        self.assertTrue(admin_role['options']['immutable'])
+        self.assertTrue(member_role['options']['immutable'])
+        self.assertTrue(reader_role['options']['immutable'])
+
+    def test_bootstrap_with_default_immutable_roles(self):
+        CONF(args=['bootstrap',
+                   '--bootstrap-password', uuid.uuid4().hex],
+             project='keystone')
+        self._do_test_bootstrap(self.bootstrap)
+        admin_role = PROVIDERS.role_api.get_role(self.bootstrap.role_id)
+        reader_role = PROVIDERS.role_api.get_role(
+            self.bootstrap.reader_role_id)
+        member_role = PROVIDERS.role_api.get_role(
+            self.bootstrap.member_role_id)
+        self.assertTrue(admin_role['options']['immutable'])
+        self.assertTrue(member_role['options']['immutable'])
+        self.assertTrue(reader_role['options']['immutable'])
+
+    def test_bootstrap_with_no_immutable_roles(self):
+        CONF(args=['bootstrap',
+                   '--bootstrap-password', uuid.uuid4().hex,
+                   '--no-immutable-roles'],
+             project='keystone')
+        self._do_test_bootstrap(self.bootstrap)
+        admin_role = PROVIDERS.role_api.get_role(self.bootstrap.role_id)
+        reader_role = PROVIDERS.role_api.get_role(
+            self.bootstrap.reader_role_id)
+        member_role = PROVIDERS.role_api.get_role(
+            self.bootstrap.member_role_id)
+        self.assertNotIn('immutable', admin_role['options'])
+        self.assertNotIn('immutable', member_role['options'])
+        self.assertNotIn('immutable', reader_role['options'])
 
     def test_bootstrap_with_ambiguous_role_names(self):
-        bootstrap = cli.BootStrap()
         # bootstrap system to create the default admin role
-        self._do_test_bootstrap(bootstrap)
+        self._do_test_bootstrap(self.bootstrap)
 
         # create a domain-specific roles that share the same names as the
         # default roles created by keystone-manage bootstrap
@@ -304,7 +364,7 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
 
             # ensure subsequent bootstrap attempts don't fail because of
             # ambiguity
-            self._do_test_bootstrap(bootstrap)
+            self._do_test_bootstrap(self.bootstrap)
 
 
 class CliBootStrapTestCaseWithEnvironment(CliBootStrapTestCase):
@@ -356,9 +416,11 @@ class CliBootStrapTestCaseWithEnvironment(CliBootStrapTestCase):
             fixtures.EnvironmentVariable('OS_BOOTSTRAP_REGION_ID',
                                          newvalue=self.region_id))
 
+        PROVIDERS.resource_api.create_domain(
+            default_fixtures.ROOT_DOMAIN['id'], default_fixtures.ROOT_DOMAIN)
+
     def test_assignment_created_with_user_exists(self):
         # test assignment can be created if user already exists.
-        bootstrap = cli.BootStrap()
         PROVIDERS.resource_api.create_domain(self.default_domain['id'],
                                              self.default_domain)
         user_ref = unit.new_user_ref(self.default_domain['id'],
@@ -366,47 +428,42 @@ class CliBootStrapTestCaseWithEnvironment(CliBootStrapTestCase):
                                      password=self.password)
         PROVIDERS.identity_api.create_user(user_ref)
 
-        self._do_test_bootstrap(bootstrap)
+        self._do_test_bootstrap(self.bootstrap)
 
     def test_assignment_created_with_project_exists(self):
         # test assignment can be created if project already exists.
-        bootstrap = cli.BootStrap()
         PROVIDERS.resource_api.create_domain(self.default_domain['id'],
                                              self.default_domain)
         project_ref = unit.new_project_ref(self.default_domain['id'],
                                            name=self.project_name)
         PROVIDERS.resource_api.create_project(project_ref['id'], project_ref)
-        self._do_test_bootstrap(bootstrap)
+        self._do_test_bootstrap(self.bootstrap)
 
     def test_assignment_created_with_role_exists(self):
         # test assignment can be created if role already exists.
-        bootstrap = cli.BootStrap()
         PROVIDERS.resource_api.create_domain(self.default_domain['id'],
                                              self.default_domain)
         role = unit.new_role_ref(name=self.role_name)
         PROVIDERS.role_api.create_role(role['id'], role)
-        self._do_test_bootstrap(bootstrap)
+        self._do_test_bootstrap(self.bootstrap)
 
     def test_assignment_created_with_region_exists(self):
         # test assignment can be created if region already exists.
-        bootstrap = cli.BootStrap()
         PROVIDERS.resource_api.create_domain(self.default_domain['id'],
                                              self.default_domain)
         region = unit.new_region_ref(id=self.region_id)
         PROVIDERS.catalog_api.create_region(region)
-        self._do_test_bootstrap(bootstrap)
+        self._do_test_bootstrap(self.bootstrap)
 
     def test_endpoints_created_with_service_exists(self):
         # test assignment can be created if service already exists.
-        bootstrap = cli.BootStrap()
         PROVIDERS.resource_api.create_domain(self.default_domain['id'],
                                              self.default_domain)
         service = unit.new_service_ref(name=self.service_name)
         PROVIDERS.catalog_api.create_service(service['id'], service)
-        self._do_test_bootstrap(bootstrap)
+        self._do_test_bootstrap(self.bootstrap)
 
     def test_endpoints_created_with_endpoint_exists(self):
-        bootstrap = cli.BootStrap()
         # test assignment can be created if endpoint already exists.
         PROVIDERS.resource_api.create_domain(self.default_domain['id'],
                                              self.default_domain)
@@ -422,7 +479,22 @@ class CliBootStrapTestCaseWithEnvironment(CliBootStrapTestCase):
                                          region_id=self.region_id)
         PROVIDERS.catalog_api.create_endpoint(endpoint['id'], endpoint)
 
-        self._do_test_bootstrap(bootstrap)
+        self._do_test_bootstrap(self.bootstrap)
+
+    def test_endpoints_created_with_new_endpoints(self):
+        service = unit.new_service_ref(name=self.service_name, type='identity')
+        PROVIDERS.catalog_api.create_service(service['id'], service)
+        region = unit.new_region_ref(id=self.region_id)
+        PROVIDERS.catalog_api.create_region(region)
+        endpoint = unit.new_endpoint_ref(interface='public',
+                                         service_id=service['id'],
+                                         url=uuid.uuid4().hex,
+                                         region_id=self.region_id)
+        PROVIDERS.catalog_api.create_endpoint(endpoint['id'], endpoint)
+
+        self._do_test_bootstrap(self.bootstrap)
+        updated_endpoint = PROVIDERS.catalog_api.get_endpoint(endpoint['id'])
+        self.assertEqual(updated_endpoint['url'], self.bootstrap.public_url)
 
 
 class CliDomainConfigAllTestCase(unit.SQLDriverOverrides, unit.TestCase):
@@ -470,6 +542,9 @@ class CliDomainConfigAllTestCase(unit.SQLDriverOverrides, unit.TestCase):
         def create_domain(domain):
             return PROVIDERS.resource_api.create_domain(domain['id'], domain)
 
+        PROVIDERS.resource_api.create_domain(
+            default_fixtures.ROOT_DOMAIN['id'], default_fixtures.ROOT_DOMAIN)
+
         self.domains = {}
         self.addCleanup(self.cleanup_domains)
         for x in range(1, self.domain_count):
@@ -568,7 +643,7 @@ class CliDomainConfigSingleDomainTestCase(CliDomainConfigAllTestCase):
         # Now try and upload the settings in the configuration file for the
         # default domain
         provider_api.ProviderAPIs._clear_registry_instances()
-        with mock.patch('six.moves.builtins.print') as mock_print:
+        with mock.patch('builtins.print') as mock_print:
             self.assertRaises(unit.UnexpectedExit, cli.DomainConfigUpload.main)
             file_name = ('keystone.%s.conf' % self.default_domain['name'])
             error_msg = _(
@@ -593,7 +668,7 @@ class CliDomainConfigNoOptionsTestCase(CliDomainConfigAllTestCase):
 
     def test_config_upload(self):
         provider_api.ProviderAPIs._clear_registry_instances()
-        with mock.patch('six.moves.builtins.print') as mock_print:
+        with mock.patch('builtins.print') as mock_print:
             self.assertRaises(unit.UnexpectedExit, cli.DomainConfigUpload.main)
             mock_print.assert_has_calls(
                 [mock.call(
@@ -610,7 +685,7 @@ class CliDomainConfigTooManyOptionsTestCase(CliDomainConfigAllTestCase):
 
     def test_config_upload(self):
         provider_api.ProviderAPIs._clear_registry_instances()
-        with mock.patch('six.moves.builtins.print') as mock_print:
+        with mock.patch('builtins.print') as mock_print:
             self.assertRaises(unit.UnexpectedExit, cli.DomainConfigUpload.main)
             mock_print.assert_has_calls(
                 [mock.call(_('The --all option cannot be used with '
@@ -627,7 +702,7 @@ class CliDomainConfigInvalidDomainTestCase(CliDomainConfigAllTestCase):
 
     def test_config_upload(self):
         provider_api.ProviderAPIs._clear_registry_instances()
-        with mock.patch('six.moves.builtins.print') as mock_print:
+        with mock.patch('builtins.print') as mock_print:
             self.assertRaises(unit.UnexpectedExit, cli.DomainConfigUpload.main)
             file_name = 'keystone.%s.conf' % self.invalid_domain_name
             error_msg = (_(
@@ -679,18 +754,28 @@ class CliDBSyncTestCase(unit.BaseTestCase):
             self.version = None
 
     def setUp(self):
-        super(CliDBSyncTestCase, self).setUp()
+        super().setUp()
         self.config_fixture = self.useFixture(oslo_config.fixture.Config(CONF))
         self.config_fixture.register_cli_opt(cli.command_opt)
-        upgrades.offline_sync_database_to_version = mock.Mock()
-        upgrades.expand_schema = mock.Mock()
-        upgrades.migrate_data = mock.Mock()
-        upgrades.contract_schema = mock.Mock()
+
+        self.patchers = patchers = [
+            mock.patch.object(upgrades, "offline_sync_database_to_version"),
+            mock.patch.object(upgrades, "expand_schema"),
+            mock.patch.object(upgrades, "migrate_data"),
+            mock.patch.object(upgrades, "contract_schema"),
+        ]
+        for p in patchers:
+            p.start()
         self.command_check = False
         self.command_expand = False
         self.command_migrate = False
         self.command_contract = False
 
+    def tearDown(self):
+        for p in self.patchers:
+            p.stop()
+        super().tearDown()
+
     def _assert_correct_call(self, mocked_function):
         for func in [upgrades.offline_sync_database_to_version,
                      upgrades.expand_schema,
@@ -729,17 +814,6 @@ class CliDBSyncTestCase(unit.BaseTestCase):
         cli.DbSync.main()
         self._assert_correct_call(upgrades.contract_schema)
 
-    @mock.patch('keystone.cmd.cli.upgrades.get_db_version')
-    def test_db_sync_check_when_database_is_empty(self, mocked_get_db_version):
-        e = migration.exception.DBMigrationError("Invalid version")
-        mocked_get_db_version.side_effect = e
-        checker = cli.DbSync()
-
-        log_info = self.useFixture(fixtures.FakeLogger(level=log.INFO))
-        status = checker.check_db_sync_status()
-        self.assertIn("not currently under version control", log_info.output)
-        self.assertEqual(status, 2)
-
 
 class TestMappingPopulate(unit.SQLDriverOverrides, unit.TestCase):
 
@@ -880,6 +954,63 @@ class CachingDoctorTests(unit.TestCase):
                                    backend='dogpile.cache.memory')
         self.assertFalse(caching.symptom_caching_enabled_without_a_backend())
 
+    @mock.patch('keystone.cmd.doctor.caching.cache.CACHE_REGION')
+    def test_symptom_connection_to_memcached(self, cache_mock):
+        self.config_fixture.config(group='cache', enabled=True)
+        self.config_fixture.config(
+            group='cache',
+            memcache_servers=['alpha.com:11211', 'beta.com:11211']
+        )
+        self.config_fixture.config(
+            group='cache', backend='dogpile.cache.memcached'
+        )
+
+        # No symptom detected: Caching driver can connect to both memcached
+        # servers
+        cache_mock.actual_backend.client.get_stats.return_value = (
+            [('alpha.com', {}), ('beta.com', {})]
+        )
+        self.assertFalse(caching.symptom_connection_to_memcached())
+
+        # Symptom detected: Caching driver can't connect to either memcached
+        # server
+        cache_mock.actual_backend.client.get_stats.return_value = []
+        self.assertTrue(caching.symptom_connection_to_memcached())
+
+        # Symptom detected: Caching driver can't connect to one memcached
+        # server
+        cache_mock.actual_backend.client.get_stats.return_value = [
+            ('alpha.com', {})
+        ]
+        self.assertTrue(caching.symptom_connection_to_memcached())
+
+        self.config_fixture.config(
+            group='cache',
+            memcache_servers=['alpha.com:11211', 'beta.com:11211']
+        )
+        self.config_fixture.config(
+            group='cache', backend='oslo_cache.memcache_pool'
+        )
+
+        # No symptom detected: Caching driver can connect to both memcached
+        # servers
+        cache_mock.actual_backend.client.get_stats.return_value = (
+            [('alpha.com', {}), ('beta.com', {})]
+        )
+        self.assertFalse(caching.symptom_connection_to_memcached())
+
+        # Symptom detected: Caching driver can't connect to either memcached
+        # server
+        cache_mock.actual_backend.client.get_stats.return_value = []
+        self.assertTrue(caching.symptom_connection_to_memcached())
+
+        # Symptom detected: Caching driver can't connect to one memcached
+        # server
+        cache_mock.actual_backend.client.get_stats.return_value = [
+            ('alpha.com', {})
+        ]
+        self.assertTrue(caching.symptom_connection_to_memcached())
+
 
 class CredentialDoctorTests(unit.TestCase):
 
@@ -1237,7 +1368,7 @@ class SecurityComplianceDoctorTests(unit.TestCase):
         # Symptom Detected: Regular expression is invalid
         self.config_fixture.config(
             group='security_compliance',
-            password_regex='^^(??=.*\d)$')
+            password_regex=r'^^(??=.*\d)$')
         self.assertTrue(
             security_compliance.symptom_invalid_password_regular_expression())
 
@@ -1245,7 +1376,7 @@ class SecurityComplianceDoctorTests(unit.TestCase):
         # No Symptom Detected: Regular expression is valid
         self.config_fixture.config(
             group='security_compliance',
-            password_regex='^(?=.*\d)(?=.*[a-zA-Z]).{7,}$')
+            password_regex=r'^(?=.*\d)(?=.*[a-zA-Z]).{7,}$')
         self.assertFalse(
             security_compliance.symptom_invalid_password_regular_expression())
 
@@ -1261,7 +1392,7 @@ class SecurityComplianceDoctorTests(unit.TestCase):
         # Symptom Detected: Regular expression is set but description is not
         self.config_fixture.config(
             group='security_compliance',
-            password_regex='^(?=.*\d)(?=.*[a-zA-Z]).{7,}$')
+            password_regex=r'^(?=.*\d)(?=.*[a-zA-Z]).{7,}$')
         self.config_fixture.config(
             group='security_compliance',
             password_regex_description=None)
@@ -1274,7 +1405,7 @@ class SecurityComplianceDoctorTests(unit.TestCase):
         desc = '1 letter, 1 digit, and a minimum length of 7 is required'
         self.config_fixture.config(
             group='security_compliance',
-            password_regex='^(?=.*\d)(?=.*[a-zA-Z]).{7,}$')
+            password_regex=r'^(?=.*\d)(?=.*[a-zA-Z]).{7,}$')
         self.config_fixture.config(
             group='security_compliance',
             password_regex_description=desc)
@@ -1648,18 +1779,292 @@ class TestGroupMappingPurgeFunctional(unit.SQLDriverOverrides, unit.TestCase):
             )
 
 
-class TestTokenFlush(unit.TestCase):
+class TestTrustFlush(unit.SQLDriverOverrides, unit.BaseTestCase):
 
-    def test_token_flush_emits_warning(self):
-        expected_msg = (
-            'The token_flush command is deprecated and no longer needed with '
-            'the development of non-persistent token formats. It will be '
-            'removed in Stein. It is recommended that you remove usage of '
-            'this command or integrate it\'s functionality into a separate '
-            'tool if you are using an out-of-tree provider that relies on '
-            'persistent token storage.'
-        )
-        logging = self.useFixture(fixtures.FakeLogger())
-        tf = cli.TokenFlush()
-        tf.main()
+    class FakeConfCommand(object):
+        def __init__(self, parent):
+            self.extension = False
+            self.project_id = parent.command_project_id
+            self.trustor_user_id = parent.command_trustor_user_id
+            self.trustee_user_id = parent.command_trustee_user_id
+            self.date = parent.command_date
+
+    def setUp(self):
+        # Set up preset cli options and a parser
+        super(TestTrustFlush, self).setUp()
+        self.useFixture(database.Database())
+        self.config_fixture = self.useFixture(oslo_config.fixture.Config(CONF))
+        self.config_fixture.register_cli_opt(cli.command_opt)
+        # For unit tests that should not throw any errors,
+        # Use the argument parser to test that the combinations work
+        parser_test = argparse.ArgumentParser()
+        subparsers = parser_test.add_subparsers()
+        self.parser = cli.TrustFlush.add_argument_parser(subparsers)
+
+    def config_files(self):
+        config_files = super(TestTrustFlush, self).config_files()
+        config_files.append(unit.dirs.tests_conf('backend_sql.conf'))
+        return config_files
+
+    def test_trust_flush(self):
+        self.command_project_id = None
+        self.command_trustor_user_id = None
+        self.command_trustee_user_id = None
+        self.command_date = datetime.datetime.utcnow()
+        self.useFixture(fixtures.MockPatchObject(
+            CONF, 'command', self.FakeConfCommand(self)))
+
+        def fake_load_backends():
+            return dict(
+                trust_api=keystone.trust.core.Manager())
+
+        self.useFixture(fixtures.MockPatch(
+            'keystone.server.backends.load_backends',
+            side_effect=fake_load_backends))
+        trust = cli.TrustFlush()
+        trust.main()
+
+    def test_trust_flush_with_invalid_date(self):
+        self.command_project_id = None
+        self.command_trustor_user_id = None
+        self.command_trustee_user_id = None
+        self.command_date = '4/10/92'
+        self.useFixture(fixtures.MockPatchObject(
+            CONF, 'command', self.FakeConfCommand(self)))
+
+        def fake_load_backends():
+            return dict(
+                trust_api=keystone.trust.core.Manager())
+
+        self.useFixture(fixtures.MockPatch(
+            'keystone.server.backends.load_backends',
+            side_effect=fake_load_backends))
+        # Clear backend dependencies, since cli loads these manually
+        provider_api.ProviderAPIs._clear_registry_instances()
+        trust = cli.TrustFlush()
+        self.assertRaises(ValueError, trust.main)
+
+
+class TestMappingEngineTester(unit.BaseTestCase):
+
+    class FakeConfCommand(object):
+        def __init__(self, parent):
+            self.extension = False
+            self.rules = parent.command_rules
+            self.input = parent.command_input
+            self.prefix = parent.command_prefix
+            self.engine_debug = parent.command_engine_debug
+
+    def setUp(self):
+        # Set up preset cli options and a parser
+        super(TestMappingEngineTester, self).setUp()
+        self.mapping_id = uuid.uuid4().hex
+        self.rules_pathname = None
+        self.rules = None
+        self.assertion_pathname = None
+        self.assertion = None
+        self.logging = self.useFixture(fixtures.LoggerFixture())
+        self.useFixture(database.Database())
+        self.config_fixture = self.useFixture(oslo_config.fixture.Config(CONF))
+        self.config_fixture.register_cli_opt(cli.command_opt)
+        # For unit tests that should not throw any erorrs,
+        # Use the argument parser to test that the combinations work
+        parser_test = argparse.ArgumentParser()
+        subparsers = parser_test.add_subparsers()
+        self.parser = cli.MappingEngineTester.add_argument_parser(subparsers)
+
+    def config_files(self):
+        config_files = super(TestMappingEngineTester, self).config_files()
+        config_files.append(unit.dirs.tests_conf('backend_sql.conf'))
+        return config_files
+
+    def test_mapping_engine_tester_with_invalid_rules_file(self):
+        tempfilejson = self.useFixture(temporaryfile.SecureTempFile())
+        tmpinvalidfile = tempfilejson.file_name
+        # Here the data required for rules should be in JSON format
+        # whereas the file contains text.
+        with open(tmpinvalidfile, 'w') as f:
+            f.write("This is an invalid data")
+        self.command_rules = tmpinvalidfile
+        self.command_input = tmpinvalidfile
+        self.command_prefix = None
+        self.command_engine_debug = True
+        self.useFixture(fixtures.MockPatchObject(
+            CONF, 'command', self.FakeConfCommand(self)))
+        mapping_engine = cli.MappingEngineTester()
+        self.assertRaises(SystemExit, mapping_engine.main)
+
+    def test_mapping_engine_tester_with_invalid_input_file(self):
+        tempfilejson = self.useFixture(temporaryfile.SecureTempFile())
+        tmpfilejsonname = tempfilejson.file_name
+        updated_mapping = copy.deepcopy(mapping_fixtures.MAPPING_SMALL)
+        with open(tmpfilejsonname, 'w') as f:
+            f.write(jsonutils.dumps(updated_mapping))
+        self.command_rules = tmpfilejsonname
+        # Here invalid.csv does not exist
+        self.command_input = "invalid.csv"
+        self.command_prefix = None
+        self.command_engine_debug = True
+        self.useFixture(fixtures.MockPatchObject(
+            CONF, 'command', self.FakeConfCommand(self)))
+        mapping_engine = cli.MappingEngineTester()
+        self.assertRaises(SystemExit, mapping_engine.main)
+
+    def test_mapping_engine_tester(self):
+        tempfilejson = self.useFixture(temporaryfile.SecureTempFile())
+        tmpfilejsonname = tempfilejson.file_name
+        updated_mapping = copy.deepcopy(mapping_fixtures.MAPPING_SMALL)
+        with open(tmpfilejsonname, 'w') as f:
+            f.write(jsonutils.dumps(updated_mapping))
+        self.command_rules = tmpfilejsonname
+        tempfile = self.useFixture(temporaryfile.SecureTempFile())
+        tmpfilename = tempfile.file_name
+        with open(tmpfilename, 'w') as f:
+            f.write("\n")
+            f.write("UserName:me\n")
+            f.write("orgPersonType:NoContractor\n")
+            f.write("LastName:Bo\n")
+            f.write("FirstName:Jill\n")
+        self.command_input = tmpfilename
+        self.command_prefix = None
+        self.command_engine_debug = True
+        self.useFixture(fixtures.MockPatchObject(
+            CONF, 'command', self.FakeConfCommand(self)))
+        mapping_engine = cli.MappingEngineTester()
+        with mock.patch('builtins.print') as mock_print:
+            mapping_engine.main()
+            self.assertEqual(mock_print.call_count, 3)
+            call = mock_print.call_args_list[0]
+            args, kwargs = call
+            self.assertTrue(args[0].startswith('Using Rules:'))
+            call = mock_print.call_args_list[1]
+            args, kwargs = call
+            self.assertTrue(args[0].startswith('Using Assertion:'))
+            call = mock_print.call_args_list[2]
+            args, kwargs = call
+            expected = {
+                "group_names": [],
+                "user": {
+                    "type": "ephemeral",
+                    "name": "me"
+                },
+                "projects": [],
+                "group_ids": ["0cd5e9"]
+            }
+            self.assertEqual(jsonutils.loads(args[0]), expected)
+
+    def test_mapping_engine_tester_with_invalid_data(self):
+        tempfilejson = self.useFixture(temporaryfile.SecureTempFile())
+        tmpfilejsonname = tempfilejson.file_name
+        updated_mapping = copy.deepcopy(mapping_fixtures.MAPPING_SMALL)
+        with open(tmpfilejsonname, 'w') as f:
+            f.write(jsonutils.dumps(updated_mapping))
+        self.command_rules = tmpfilejsonname
+        tempfile = self.useFixture(temporaryfile.SecureTempFile())
+        tmpfilename = tempfile.file_name
+        # Here we do not have any value matching to type 'Email'
+        # and condition in mapping_engine_test_rules.json
+        with open(tmpfilename, 'w') as f:
+            f.write("\n")
+            f.write("UserName: me\n")
+            f.write("Email: No@example.com\n")
+        self.command_input = tmpfilename
+        self.command_prefix = None
+        self.command_engine_debug = True
+        self.useFixture(fixtures.MockPatchObject(
+            CONF, 'command', self.FakeConfCommand(self)))
+        mapping_engine = cli.MappingEngineTester()
+        self.assertRaises(exception.ValidationError,
+                          mapping_engine.main)
+
+    def test_mapping_engine_tester_logs_direct_maps(self):
+        tempfilejson = self.useFixture(temporaryfile.SecureTempFile())
+        tmpfilejsonname = tempfilejson.file_name
+        updated_mapping = copy.deepcopy(mapping_fixtures.MAPPING_SMALL)
+        with open(tmpfilejsonname, 'w') as f:
+            f.write(jsonutils.dumps(updated_mapping))
+        self.command_rules = tmpfilejsonname
+        tempfile = self.useFixture(temporaryfile.SecureTempFile())
+        tmpfilename = tempfile.file_name
+        with open(tmpfilename, 'w') as f:
+            f.write("\n")
+            f.write("UserName:me\n")
+            f.write("orgPersonType:NoContractor\n")
+            f.write("LastName:Bo\n")
+            f.write("FirstName:Jill\n")
+        self.command_input = tmpfilename
+        self.command_prefix = None
+        self.command_engine_debug = True
+        self.useFixture(fixtures.MockPatchObject(
+            CONF, 'command', self.FakeConfCommand(self)))
+        mapping_engine = cli.MappingEngineTester()
+        logging = self.useFixture(fixtures.FakeLogger(level=log.DEBUG))
+        mapping_engine.main()
+        expected_msg = "direct_maps: [['me']]"
         self.assertThat(logging.output, matchers.Contains(expected_msg))
+
+
+class CliStatusTestCase(unit.SQLDriverOverrides, unit.TestCase):
+
+    def setUp(self):
+        self.useFixture(database.Database())
+        super(CliStatusTestCase, self).setUp()
+        self.load_backends()
+        self.policy_file = self.useFixture(temporaryfile.SecureTempFile())
+        self.policy_file_name = self.policy_file.file_name
+        self.useFixture(
+            policy.Policy(
+                self.config_fixture, policy_file=self.policy_file_name
+            )
+        )
+        self.checks = status.Checks()
+
+    def test_check_safe_trust_policies(self):
+        with open(self.policy_file_name, 'w') as f:
+            overridden_policies = {
+                'identity:list_trusts': '',
+                'identity:delete_trust': '',
+                'identity:get_trust': '',
+                'identity:list_roles_for_trust': '',
+                'identity:get_role_for_trust': ''
+            }
+            f.write(jsonutils.dumps(overridden_policies))
+        result = self.checks.check_trust_policies_are_not_empty()
+        self.assertEqual(upgradecheck.Code.FAILURE, result.code)
+        with open(self.policy_file_name, 'w') as f:
+            overridden_policies = {
+                'identity:list_trusts': 'rule:admin_required',
+                'identity:delete_trust': 'rule:admin_required',
+                'identity:get_trust': 'rule:admin_required',
+                'identity:list_roles_for_trust': 'rule:admin_required',
+                'identity:get_role_for_trust': 'rule:admin_required'
+            }
+            f.write(jsonutils.dumps(overridden_policies))
+        result = self.checks.check_trust_policies_are_not_empty()
+        self.assertEqual(upgradecheck.Code.SUCCESS, result.code)
+        with open(self.policy_file_name, 'w') as f:
+            overridden_policies = {}
+            f.write(jsonutils.dumps(overridden_policies))
+        result = self.checks.check_trust_policies_are_not_empty()
+        self.assertEqual(upgradecheck.Code.SUCCESS, result.code)
+
+    def test_check_immutable_roles(self):
+        role_ref = unit.new_role_ref(name='admin')
+        PROVIDERS.role_api.create_role(role_ref['id'], role_ref)
+        result = self.checks.check_default_roles_are_immutable()
+        self.assertEqual(upgradecheck.Code.FAILURE, result.code)
+        role_ref['options'] = {'immutable': True}
+        PROVIDERS.role_api.update_role(role_ref['id'], role_ref)
+        result = self.checks.check_default_roles_are_immutable()
+        self.assertEqual(upgradecheck.Code.SUCCESS, result.code)
+        # Check domain-specific roles are not reported
+        PROVIDERS.resource_api.create_domain(
+            default_fixtures.ROOT_DOMAIN['id'],
+            default_fixtures.ROOT_DOMAIN)
+        domain_ref = unit.new_domain_ref()
+        domain = PROVIDERS.resource_api.create_domain(
+            domain_ref['id'], domain_ref)
+        role_ref = unit.new_role_ref(name='admin', domain_id=domain['id'])
+        PROVIDERS.role_api.create_role(role_ref['id'], role_ref)
+        result = self.checks.check_default_roles_are_immutable()
+        self.assertEqual(upgradecheck.Code.SUCCESS, result.code)
