@@ -13,12 +13,11 @@
 # under the License.
 
 import datetime
-import uuid
-
 import http.client
 import oslo_context.context
 from oslo_serialization import jsonutils
 from testtools import matchers
+import uuid
 import webtest
 
 from keystone.common import authorization
@@ -247,7 +246,7 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
             domain_id=DEFAULT_DOMAIN_ID)
         self.default_domain_user_id = self.default_domain_user['id']
 
-        # create & grant policy.json's default role for admin_required
+        # create & grant policy.yaml's default role for admin_required
         self.role = unit.new_role_ref(name='admin')
         self.role_id = self.role['id']
         PROVIDERS.role_api.create_role(self.role_id, self.role)
@@ -1101,7 +1100,7 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
         if ref:
             links = ref.pop('links')
             try:
-                self.assertDictContainsSubset(ref, entity)
+                self.assertLessEqual(ref.items(), entity.items())
                 self.assertIn(links['assignment'],
                               entity['links']['assignment'])
             finally:
@@ -1238,6 +1237,13 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
         return environment
 
 
+class OAuth2RestfulTestCase(RestfulTestCase):
+    def assertValidErrorResponse(self, response):
+        resp = response.result
+        self.assertIsNotNone(resp.get('error'))
+        self.assertIsNotNone(resp.get('error_description'))
+
+
 class VersionTestCase(RestfulTestCase):
     def test_get_version(self):
         pass
