@@ -17,9 +17,8 @@ import copy
 import functools
 import random
 
-import mock
+import http.client
 from oslo_serialization import jsonutils
-from six.moves import http_client
 from testtools import matchers as tt_matchers
 import webob
 
@@ -37,9 +36,9 @@ v3_MEDIA_TYPES = [
 ]
 
 v3_EXPECTED_RESPONSE = {
-    "id": "v3.11",
+    "id": "v3.14",
     "status": "stable",
-    "updated": "2018-10-15T00:00:00Z",
+    "updated": "2020-04-07T00:00:00Z",
     "links": [
         {
             "rel": "self",
@@ -142,9 +141,14 @@ FEDERATED_IDP_SPECIFIC_WEBSSO = ('/auth/OS-FEDERATION/identity_providers/'
 
 APPLICATION_CREDENTIAL = ('/users/{user_id}/application_credentials/'
                           '{application_credential_id}')
+APPLICATION_CREDENTIALS = '/users/{user_id}/application_credentials'
 APPLICATION_CREDENTIAL_RELATION = (
     json_home.build_v3_parameter_relation('application_credential_id'))
 
+ACCESS_RULE = '/users/{user_id}/access_rules/{access_rule_id}'
+ACCESS_RULES = '/users/{user_id}/access_rules'
+ACCESS_RULE_RELATION = json_home.build_v3_parameter_relation('access_rule_id')
+
 V3_JSON_HOME_RESOURCES = {
     json_home.build_v3_resource_relation('auth_tokens'): {
         'href': '/auth/tokens'},
@@ -367,6 +371,9 @@ V3_JSON_HOME_RESOURCES = {
         'href-template': '/users/{user_id}/projects',
         'href-vars': {'user_id': json_home.Parameters.USER_ID, }},
     json_home.build_v3_resource_relation('users'): {'href': '/users'},
+    json_home.build_v3_extension_resource_relation(
+        'OS-OAUTH2', '1.0', 'token'): {
+        'href': '/OS-OAUTH2/token'},
     _build_federation_rel(resource_name='domains'): {
         'href': '/auth/domains'},
     _build_federation_rel(resource_name='websso'): {
@@ -391,7 +398,7 @@ V3_JSON_HOME_RESOURCES = {
     {
         'href-template': '/OS-FEDERATION/identity_providers/{idp_id}',
         'href-vars': {'idp_id': IDP_ID_PARAMETER_RELATION, }},
-    _build_federation_rel(resource_name='identity_providers'): {
+    _build_federation_rel(resource_name='identity_providers_websso'): {
         'href-template': FEDERATED_IDP_SPECIFIC_WEBSSO,
         'href-vars': {
             'idp_id': IDP_ID_PARAMETER_RELATION,
@@ -633,11 +640,24 @@ V3_JSON_HOME_RESOURCES = {
         'href': '/limits/model',
         'hints': {'status': 'experimental'}
     },
+    json_home.build_v3_resource_relation('application_credentials'): {
+        'href-template': APPLICATION_CREDENTIALS,
+        'href-vars': {
+            'user_id': json_home.build_v3_parameter_relation('user_id')}},
     json_home.build_v3_resource_relation('application_credential'): {
         'href-template': APPLICATION_CREDENTIAL,
         'href-vars': {
             'application_credential_id': APPLICATION_CREDENTIAL_RELATION,
-            'user_id': json_home.build_v3_parameter_relation('user_id')}}
+            'user_id': json_home.build_v3_parameter_relation('user_id')}},
+    json_home.build_v3_resource_relation('access_rules'): {
+        'href-template': ACCESS_RULES,
+        'href-vars': {
+            'user_id': json_home.build_v3_parameter_relation('user_id')}},
+    json_home.build_v3_resource_relation('access_rule'): {
+        'href-template': ACCESS_RULE,
+        'href-vars': {
+            'access_rule_id': ACCESS_RULE_RELATION,
+            'user_id': json_home.build_v3_parameter_relation('user_id')}},
 }
 
 
@@ -738,7 +758,7 @@ class VersionTestCase(unit.TestCase):
     def test_public_version_v3(self):
         client = TestClient(self.public_app)
         resp = client.get('/v3/')
-        self.assertEqual(http_client.OK, resp.status_int)
+        self.assertEqual(http.client.OK, resp.status_int)
         data = jsonutils.loads(resp.body)
         expected = v3_VERSION_RESPONSE
         self._paste_in_port(expected['version'],
@@ -750,13 +770,12 @@ class VersionTestCase(unit.TestCase):
         for app in (self.public_app,):
             client = TestClient(app)
             resp = client.get('/v3/')
-            self.assertEqual(http_client.OK, resp.status_int)
+            self.assertEqual(http.client.OK, resp.status_int)
             data = jsonutils.loads(resp.body)
             expected = v3_VERSION_RESPONSE
             self._paste_in_port(expected['version'], 'http://localhost/v3/')
             self.assertEqual(expected, data)
 
-    @mock.patch.object(discovery, '_VERSIONS', ['v3'])
     def test_v2_disabled(self):
         # NOTE(morgan): This test should be kept, v2.0 is removed and should
         # never return, this prevents regression[s]/v2.0 discovery doc
@@ -770,7 +789,7 @@ class VersionTestCase(unit.TestCase):
 
         # request to /v3 should pass
         resp = client.get('/v3/')
-        self.assertEqual(http_client.OK, resp.status_int)
+        self.assertEqual(http.client.OK, resp.status_int)
         data = jsonutils.loads(resp.body)
         expected = v3_VERSION_RESPONSE
         self._paste_in_port(expected['version'],
@@ -799,9 +818,14 @@ class VersionTestCase(unit.TestCase):
         self.assertThat(resp.status, tt_matchers.Equals('200 OK'))
         self.assertThat(resp.headers['Content-Type'],
                         tt_matchers.Equals('application/json-home'))
-
-        self.assertThat(jsonutils.loads(resp.body),
-                        tt_matchers.Equals(exp_json_home_data))
+        maxDiff = self.maxDiff
+        self.maxDiff = None
+        # NOTE(morgan): Changed from tt_matchers.Equals to make it easier to
+        # determine issues. Reset maxDiff to the original value at the end
+        # of the assert.
+        self.assertDictEqual(exp_json_home_data,
+                             jsonutils.loads(resp.body))
+        self.maxDiff = maxDiff
 
     def test_json_home_v3(self):
         # If the request is /v3 and the Accept header is application/json-home
@@ -864,13 +888,6 @@ class VersionTestCase(unit.TestCase):
         # If request some unknown mime-type, get JSON.
         self.assertThat(make_request(self.getUniqueString()), JSON_MATCHER)
 
-    @mock.patch.object(discovery, '_VERSIONS', [])
-    def test_no_json_home_document_returned_when_v3_disabled(self):
-        json_home_document = discovery._v3_json_home_content()
-        json_home.translate_urls(json_home_document, '/v3')
-        expected_document = {'resources': {}}
-        self.assertEqual(expected_document, json_home_document)
-
 
 class VersionSingleAppTestCase(unit.TestCase):
     """Test running with a single application loaded.
@@ -904,10 +921,18 @@ class VersionSingleAppTestCase(unit.TestCase):
         self.assertEqual(300, resp.status_int)
         data = jsonutils.loads(resp.body)
         expected = VERSIONS_RESPONSE
+        url_with_port = 'http://localhost:%s/v3/' % self.public_port
         for version in expected['versions']['values']:
+            # TODO(morgan): Eliminate the need to do the "paste-in-port" part
+            # of the tests. Ultimately, this is very hacky and shows we are
+            # not setting up the test case sanely.
             if version['id'].startswith('v3'):
                 self._paste_in_port(
-                    version, 'http://localhost:%s/v3/' % self.public_port)
+                    version, url_with_port)
+        # Explicitly check that a location header is set and it is pointing
+        # to v3 (The preferred location for now)!
+        self.assertIn('Location', resp.headers)
+        self.assertEqual(url_with_port, resp.headers['Location'])
         self.assertThat(data, _VersionsEqual(expected))
 
     def test_public(self):
@@ -915,50 +940,3 @@ class VersionSingleAppTestCase(unit.TestCase):
 
     def test_admin(self):
         self._test_version('admin')
-
-
-class VersionBehindSslTestCase(unit.TestCase):
-    def setUp(self):
-        super(VersionBehindSslTestCase, self).setUp()
-        self.load_backends()
-        self.public_app = self.loadapp('public')
-
-    def config_overrides(self):
-        super(VersionBehindSslTestCase, self).config_overrides()
-        self.config_fixture.config(
-            secure_proxy_ssl_header='HTTP_X_FORWARDED_PROTO')
-
-    def _paste_in_port(self, response, port):
-        for link in response['links']:
-            if link['rel'] == 'self':
-                link['href'] = port
-
-    def _get_expected(self, host):
-        expected = VERSIONS_RESPONSE
-        for version in expected['versions']['values']:
-            if version['id'].startswith('v3'):
-                self._paste_in_port(version, host + 'v3/')
-        return expected
-
-    def test_versions_without_headers(self):
-        client = TestClient(self.public_app)
-        host_name = 'host-%d' % random.randint(10, 30)
-        host_port = random.randint(10000, 30000)
-        host = 'http://%s:%s/' % (host_name, host_port)
-        resp = client.get(host)
-        self.assertEqual(300, resp.status_int)
-        data = jsonutils.loads(resp.body)
-        expected = self._get_expected(host)
-        self.assertThat(data, _VersionsEqual(expected))
-
-    def test_versions_with_header(self):
-        client = TestClient(self.public_app)
-        host_name = 'host-%d' % random.randint(10, 30)
-        host_port = random.randint(10000, 30000)
-        resp = client.get('http://%s:%s/' % (host_name, host_port),
-                          headers={'X-Forwarded-Proto': 'https'})
-        self.assertEqual(300, resp.status_int)
-        data = jsonutils.loads(resp.body)
-        expected = self._get_expected('https://%s:%s/' % (host_name,
-                                                          host_port))
-        self.assertThat(data, _VersionsEqual(expected))
